@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Any, Dict, List
+from typing import Any, Dict, List, cast
 
 import click
 
@@ -13,7 +13,6 @@ from robotcode.robot.config.loader import (
     DiscoverdBy,
     load_robot_config_from_path,
 )
-from robotcode.robot.config.model import EvaluationError, RobotProfile
 from robotcode.robot.config.utils import get_config_files
 
 
@@ -44,11 +43,11 @@ def show(app: Application, no_evaluate: bool, paths: List[Path]) -> None:
         config_files, _, _ = get_config_files(paths, app.config.config_files, verbose_callback=app.verbose)
 
         config = load_robot_config_from_path(*config_files).combine_profiles(
-            *(app.config.profiles or []), verbose_callback=app.verbose
+            *(app.config.profiles or []), verbose_callback=app.verbose, error_callback=app.error
         )
 
         if not no_evaluate:
-            config = config.evaluated()
+            config = config.evaluated_with_env(verbose_callback=app.verbose, error_callback=app.error)
 
         app.print_data(
             config,
@@ -67,34 +66,42 @@ def show(app: Application, no_evaluate: bool, paths: List[Path]) -> None:
     nargs=-1,
     required=False,
 )
+@click.option("-h", "--show-hidden", is_flag=True, default=False, help="Show hidden profiles.")
+@click.option("-sp", "--sort-by-precedence", is_flag=True, default=False, help="Sort by precedence.")
 @pass_application
-def list(app: Application, paths: List[Path]) -> None:
+def list(app: Application, paths: List[Path], show_hidden: bool = False, sort_by_precedence: bool = False) -> None:
     """Lists the defined profiles in the current configuration."""
 
     try:
         config_files, _, discovered_by = get_config_files(paths, app.config.config_files, verbose_callback=app.verbose)
 
         config = load_robot_config_from_path(*config_files)
-        selected_profiles = [
-            k for k in config.select_profiles(*(app.config.profiles or []), verbose_callback=app.verbose).keys()
-        ]
 
-        def check_enabled(name: str, profile: RobotProfile) -> bool:
-            try:
-                return profile.enabled is None or bool(profile.enabled)
-            except EvaluationError as e:
-                raise ValueError(f"Cannot evaluate profile '{name}'.enabled: {e}") from e
+        _, selected_profiles, enabled_names = config.combine_profiles_ex(
+            *(app.config.profiles or []), verbose_callback=app.verbose, error_callback=app.error
+        )
+
+        selected_names = [k for k in selected_profiles.keys()]
 
         result: Dict[str, Any] = {
-            "profiles": [
-                {
-                    "name": k,
-                    "enabled": check_enabled(k, v),
-                    "description": v.description or "",
-                    "selected": True if k in selected_profiles else False,
-                }
-                for k, v in (config.profiles or {}).items()
-            ]
+            "profiles": sorted(
+                [
+                    {
+                        "name": k,
+                        "enabled": k in enabled_names,
+                        "description": v.description or "",
+                        "selected": True if k in selected_names else False,
+                        "precedence": v.precedence,
+                    }
+                    for k, v in (config.profiles or {}).items()
+                    if show_hidden or not k.startswith("_") and not v.hidden
+                ],
+                key=(
+                    (lambda v: cast(Any, str(v.get("name", ""))))
+                    if not sort_by_precedence
+                    else (lambda v: cast(Any, v.get("precedence", 0) or 0))
+                ),
+            ),
         }
 
         messages = []
@@ -125,17 +132,18 @@ def list(app: Application, paths: List[Path]) -> None:
                 *(len(profile["description"]) for profile in result["profiles"]),
             )
             header += (
-                f'| Active | Selected | Enabled | Name{(max_name - len("Name")) * " "} '
+                f'| Active | Selected | Enabled | Precedence | Name{(max_name - len("Name")) * " "} '
                 f'| Description{(max_description - len("Description")) * " "} |\n'
             )
-            header += f"|:------:|:--------:|:-------:|:{max_name * '-'}-|:{max_description * '-'}-|\n"
-            for selected, enabled, name, description in (
-                (v["selected"], v["enabled"], v["name"], v["description"]) for v in result["profiles"]
+            header += f"|:------:|:------:|:--------:|:-------:|:{max_name * '-'}-|:{max_description * '-'}-|\n"
+            for selected_profiles, enabled, name, description, precedence in (
+                (v["selected"], v["enabled"], v["name"], v["description"], v["precedence"]) for v in result["profiles"]
             ):
                 header += (
-                    f'|   {"*" if selected and enabled else " "}    '
-                    f'|    {"*" if selected else " "}     '
+                    f'|   {"*" if selected_profiles and enabled else " "}    '
+                    f'|    {"*" if selected_profiles else " "}     '
                     f'|    {"*" if enabled else " "}    '
+                    f'|    {precedence if precedence else " "}    '
                     f'| {name}{(max_name - len(name)) * " "} '
                     f'| {description if description else ""}{(max_description - len(description)) * " "} |\n'
                 )
